/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
#ifndef _DT_BINDINGS_POWER_MT6795_POWER_H
#define _DT_BINDINGS_POWER_MT6795_POWER_H

#define MT6795_POWER_DOMAIN_MM		0
#define MT6795_POWER_DOMAIN_VDEC	1
#define MT6795_POWER_DOMAIN_VENC	2
#define MT6795_POWER_DOMAIN_ISP		3
#define MT6795_POWER_DOMAIN_MJC		4
#define MT6795_POWER_DOMAIN_AUDIO	5
#define MT6795_POWER_DOMAIN_MFG_ASYNC	6
#define MT6795_POWER_DOMAIN_MFG_2D	7
#define MT6795_POWER_DOMAIN_MFG		8
#define MT6795_POWER_DOMAIN_MODEM	9

#endif /* _DT_BINDINGS_POWER_MT6795_POWER_H */
