/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_BINDINGS_MICROCHIP_PDMC_H__
#define __DT_BINDINGS_MICROCHIP_PDMC_H__

/* PDM microphone's pin placement */
#define MCHP_PDMC_DS0 0
#define MCHP_PDMC_DS1 1

/* PDM microphone clock edge sampling */
#define MCHP_PDMC_CLK_POSITIVE 0
#define MCHP_PDMC_CLK_NEGATIVE 1

#endif /* __DT_BINDINGS_MICROCHIP_PDMC_H__ */
