/* SPDX-License-Identifier: (GPL-2.0+ or MIT) */
/*
 * Copyright (c) 2019 BayLibre, SAS
 * Author: Neil Armstrong <narmstrong@baylibre.com>
 */

#ifndef _DT_BINDINGS_MESON_SM1_POWER_H
#define _DT_BINDINGS_MESON_SM1_POWER_H

#define PWRC_SM1_VPU_ID		0
#define PWRC_SM1_NNA_ID		1
#define PWRC_SM1_USB_ID		2
#define PWRC_SM1_PCIE_ID	3
#define PWRC_SM1_GE2D_ID	4
#define PWRC_SM1_AUDIO_ID	5
#define PWRC_SM1_ETH_ID		6

#endif
